package arm

import "git.urbach.dev/cli/q/src/cpu"

// addRegisterNumber performs addition or subtraction on the given register
// and optionally updates the condition flags based on the result.
func addRegisterNumber(op uint32, flags uint32, destination cpu.Register, source cpu.Register, number int) uint32 {
	number &= 0b1111_1111_1111
	return op<<30 | flags<<29 | 0b100010<<23 | (uint32(number) << 10) | (uint32(source) << 5) | uint32(destination)
}

// addRegisterRegister performs addition or subtraction on the given registers
// and optionally updates the condition flags based on the result.
func addRegisterRegister(op uint32, flags uint32, destination cpu.Register, source cpu.Register, operand cpu.Register) uint32 {
	return op<<30 | flags<<29 | 0b01011000<<21 | reg3(destination, source, operand)
}

// memory encodes an instruction with a register, a base register and an offset.
func memory(destination cpu.Register, base cpu.Register, offset int) uint32 {
	return uint32(offset&0b1_1111_1111)<<12 | uint32(base)<<5 | uint32(destination)
}

// pair encodes an instruction using a register pair with memory.
func pair(reg1 cpu.Register, reg2 cpu.Register, base cpu.Register, offset int) uint32 {
	return uint32(offset&0b111_1111)<<15 | uint32(reg2)<<10 | uint32(base)<<5 | uint32(reg1)
}

// reg3 encodes an instruction with 3 registers.
func reg3(d cpu.Register, n cpu.Register, m cpu.Register) uint32 {
	return uint32(m)<<16 | uint32(n)<<5 | uint32(d)
}

// reg4 encodes an instruction with 4 registers.
func reg4(d cpu.Register, n cpu.Register, m cpu.Register, a cpu.Register) uint32 {
	return uint32(m)<<16 | uint32(a)<<10 | uint32(n)<<5 | uint32(d)
}
